/**
* @file   tier1_testc.h
* @author Armando Miraglia (arma@lamortenera.it)
* @date   July, 2011
* @brief  implementation of the third test suite of the Tier1
*/

#include <stdlib.h>
#include <assert.h>
#include <limits.h>
#include <stdio.h>

#include "../../simclist.h"
#include "../lib/random.h"
#include "../lib/helpers.h"
#include "tier1_testc.h"

/* compare helper function */
int int_bigger_than(const void *a, const void *b);

/* implementation Test C */
void testc_check(size_t const n) {
  int valuesa[n], valuesb[n];
  list_t lista, listb, dest;
  int *el;
  int i;
  int max, min, ret;

  // --------------
  // INITIALIZATION
  // set also the comparator and the meter attributes
  // needed later on for checking the functionalities
  // --------------
  list_init(&lista);
  list_init(&listb);

  // set the comparator attribute
  ret = list_attributes_comparator(&lista, int_bigger_than);
  assert(ret == 0);
  ret = list_attributes_comparator(&listb, int_bigger_than);
  assert(ret == 0);

  // set list_attribute_copy for creating new memory areas 
  // and check the correctness of execution.
  ret = list_attributes_copy(&lista, int_size, 1);
  assert(ret == 0);

  // ---------
  // EMPTINESS
  // check if the list is empty
  // ---------
  assert(list_empty(&lista) != 0);

  // ----------
  // MIN && MAX
  // the lista is also populated while storing
  // the max value
  // ----------

  max = INT_MIN;
  min = INT_MAX;

  // populate the list and check the value added in the list
  // while doing this store also the maximum and the minimum
  for(i=0; i<n; ++i) {
    randomi(&valuesa[i]);
    ret = list_insert_at(&lista, &valuesa[i], i);
    assert(ret > 0);
    el = (int *)list_get_at(&lista, i);
    assert(valuesa[i] == *el);

    if(max < *el) max = *el;

    if(min > *el) min = *el;
  }

  // check the minimum
  if((int *)list_get_min(&lista) != NULL)
    assert(min == *(int *)list_get_min(&lista));

  // check the maximum
  if((int *)list_get_min(&lista) != NULL)
    assert(max == *(int *)list_get_max(&lista));

  // ---------
  // ITERATORS
  // ---------

  // check the elements using the iterators
  list_iterator_start(&lista);
  for(i=0; i<n && list_iterator_hasnext(&lista); ++i) {
    assert(valuesa[i] == *(int*)list_iterator_next(&lista));
  }
  assert(n==list_size(&lista)  &&  !list_iterator_hasnext(&lista));
  list_iterator_stop(&lista);

  // ------
  // CONCAT
  // The concatenation is performed using the same list.
  // The concat is stored in the dest list.
  // ------

  // populate the second list
  for(i=0; i<n; ++i) {
    randomi(&valuesb[i]);
    ret = list_insert_at(&listb, &valuesb[i], i);
  }

  // concatenate the lista with itself
  ret = list_concat(&lista, &lista, &dest);
  assert(ret == 0);
  assert(2*n == list_size(&dest));

  for(i=0; i<2*n; ++i) {
    assert(valuesa[i % n] == *(int *)list_get_at(&dest, i));
  }

  // ------------
  // DELETE RANGE
  // deleting the second half of the destination list and then
  // clearing the list.
  // ------------
  if(list_size(&dest) != 0) {
    ret = list_delete_range(&dest, n, 2*n-1);
    assert(ret == 0);
    assert(list_size(&dest) == n);

    ret = list_clear(&dest);
    assert(ret == 0);
    assert(list_size(&dest) == 0);
  }

  // ------
  // CONCAT
  // The concatenation is performed using lista and listb.
  // The concat is stored in the dest list.
  // ------

  // concatenate the lista with listb
  ret = list_concat(&lista, &listb, &dest);
  assert(ret == 0);
  assert(2*n == list_size(&dest));

  for(i=0; i<n; ++i) {
    assert(valuesa[i] == *(int *)list_get_at(&dest, i));
  }

  for(; i<2*n; ++i) {
    assert(valuesb[i % n] == *(int *)list_get_at(&dest, i));
  }

  // ------------
  // DELETE RANGE
  // deleting the first half of the destination list and then
  // clearing the list.
  // ------------
  if(list_size(&dest) != 0) {
    ret = list_delete_range(&dest, 0, n-1);
    assert(ret == 0);
    assert(list_size(&dest) == n);

    ret = list_clear(&dest);
    assert(ret == 0);
    assert(list_size(&dest) == 0);
  }

  // ----
  // SORT
  // ----

  // sort and check that the order is as expected
  // sort the array
  qsort(&valuesa[0], n, sizeof(int), int_bigger_than);

  // a) small to big
  ret = list_sort(&lista, 1);
  assert(ret == 0);

  for(i=0; i<n; ++i) {
    assert(valuesa[n - 1 - i] == *(int *)list_get_at(&lista, i));
  }

  // b) big to small
  ret = list_sort(&lista, -1);
  assert(ret == 0);

  for(i=0; i<n; ++i) {
    assert(valuesa[i] == *(int *)list_get_at(&lista, i));
  }

  // -------
  // DESTROY
  // -------

  // clean memory
  list_destroy(&lista);
  list_destroy(&listb);
  list_destroy(&dest);
}

int int_bigger_than(const void *a, const void *b) {
  return( *(int *)a < *(int *)b) - (*(int *)a > *(int *)b);
}
